"""
Base database connection.
"""

import asyncio
from abc import ABC, abstractmethod
from collections.abc import AsyncGenerator
from typing import Any, List, Optional, Tuple, Union

from sqlalchemy import Sequence, sql


class AbstractDBConnection(ABC):
    """
    Wrapper for a db connection

    Attributes:
        connection: session pool
        loop: current loop
    """

    def __init__(self, connection: Any):
        """
        Initialize session pool
        Args:
            connection: driver connection
        """
        self.loop = asyncio.get_running_loop()
        self.connection = connection

    @abstractmethod
    async def execute(self, statement: sql.ClauseElement, data: Optional[Any] = None) -> int:
        """
        Execute statement (for insert, update, delete)

        Args:
            statement: sqlalchemy statement
            data: statement data (if exists)

        Returns:
            count of affected rows (there is no certainty)
        """

    @abstractmethod
    async def fetchone(self, statement: sql.ClauseElement) -> Union[Tuple, None]:
        """
        Fetch one row

        Args:
            statement: sqlalchemy statement

        Returns:
            row  if it exists otherwise None
        """

    @abstractmethod
    async def scalar(self, statement: sql.ClauseElement) -> Union[int, str, float, bytes, None]:
        """
        Scalar value or None

        Args:
            statement: sqlalchemy statement

        Returns:
            row  if it exists otherwise None
        """

    @abstractmethod
    async def nextVal(self, sequence: Sequence) -> int:
        """
        Get next value of sequence.

        Args:
            sequence: sqlalchemy sequence

        Returns:
            next value
        """

    @abstractmethod
    async def fetchall(self, statement: sql.ClauseElement) -> List[Tuple]:
        """
        Fetch rows.

        Args:
            statement: sqlalchemy statement

        Returns:
            fetching rows
        """

    @abstractmethod
    async def stream(self, statement: sql.ClauseElement) -> AsyncGenerator[list[tuple], None]:
        """
        Fetch rows with cursor.

        Args:
            statement: sqlalchemy statement

        Yields:
            fetching rows
        """
