"""Clickhouse monitoring adaptor"""

from dataclasses import dataclass
from datetime import datetime
from typing import Iterable, List

import aiohttp
import ujson
from aiochclient import ChClient, ChClientError

from .base_adapter import BaseMonitoringAdapter
from .clickhouse_query_helpers import (
    BASIC_SERIES,
    BIN_SIZE,
    CREATE_TABLE_QUERY,
    INSERT_QUERY,
    MAX_MEANINGFUL_IMAGE_SIZE,
    NON_ESTIMATED_SERIES,
    aggregations,
)
from .points import BaseMonitoringPoint  # pylint: disable-msg=C0413

SERIES_CACHE = {}


@dataclass
class ClickhouseSettings:
    """Clickhouse monitoring settings"""

    url: str
    user: str
    password: str | None
    database: str


class ClickhouseMonitoringAdaptor(BaseMonitoringAdapter):
    """
    Clickhouse adaptor. Suspended to send points to a clickhouse
    """

    def __init__(self, settings: ClickhouseSettings, flushingPeriod: int):
        super().__init__(flushingPeriod=flushingPeriod)
        self._settings = settings
        self._buffer: dict[str, list[dict]] = {}
        self.session = aiohttp.ClientSession()
        self.client: None | ChClient = None
        self.newSeries: List[str] = []

    async def initializeMonitoring(self) -> None:
        """
        Initialize monitoring.
        """
        self.client = ChClient(
            self.session,
            url=self._settings.url,
            password=self._settings.password,
            user=self._settings.user,
            allow_experimental_object_type=1,
            database=self._settings.database,
            json=ujson,
        )
        await super().initializeMonitoring()

    async def createAggregationTables(self, series: str) -> None:
        """
        Create aggregation tables and materialized views for daily statistics
        Args:
            series (str): series name
        """
        for aggregation in aggregations.get(series, []):
            await self.client.execute(aggregation.createTable)
            await self.client.execute(aggregation.createMaterializedView)
        if series not in BASIC_SERIES:
            aggregation = aggregations.get("batch_size")
            await self.client.execute(aggregation.createTable)
            await self.client.execute(aggregation.createMaterializedView.format(fromTable=series))
        if series not in NON_ESTIMATED_SERIES:
            aggregation = aggregations.get("execution_time")
            await self.client.execute(aggregation.createTable)
            await self.client.execute(aggregation.createMaterializedView.format(fromTable=series))

    async def createFaceDetectionHeightAggregations(self, series: str, seriesPoints: list[dict]):
        """
        Create aggregation tables and materialized views for `face_detection_height` daily statistics
        Args:
            series (str): series name
            seriesPoints (list[dict]): list of points dicts
        """
        if seriesPoints and hasattr(seriesPoints[0], "face_detection_height"):
            aggregation = aggregations.get("face_detection_height")
            await self.client.execute(aggregation.createTable)
            await self.client.execute(
                aggregation.createMaterializedView.format(
                    fromTable=series, binSize=BIN_SIZE, maxHeight=MAX_MEANINGFUL_IMAGE_SIZE
                )
            )

    async def _writePoints(self, points: dict[str, list[dict]]):
        """
        Write points.
        Args:
            points: monitoring points to write
        """
        try:
            newSeries = self.newSeries.copy()
            self.newSeries = []
            for series in newSeries:
                await self.client.execute(CREATE_TABLE_QUERY.format(table=series))
                await self.createAggregationTables(series)
                SERIES_CACHE[series] = INSERT_QUERY.format(table=series)
            for series, seriesPoints in points.items():
                await self.createFaceDetectionHeightAggregations(series, seriesPoints)
                await self.client.execute(SERIES_CACHE[series], *seriesPoints)
        except ChClientError as exc:
            self.logger.warning(f"failed write monitoring buffering points, message: {exc}")

    def addPointsToBuffer(self, points: Iterable[BaseMonitoringPoint]) -> None:
        """
        Add points to buffer.

        Args:
            points: points
        """
        for point in points:
            self._buffer.setdefault(point.series, []).append(self.generateRecord(point))
            if point.series not in SERIES_CACHE and point.series not in self.newSeries:
                self.newSeries.append(point.series)

    def clearBuffer(self) -> dict[str, list[dict]]:
        """Clear monitoring buffer"""
        copyPoints = self._buffer.copy()
        self._buffer = {}
        return copyPoints

    def generateRecord(self, point: BaseMonitoringPoint) -> dict:
        """
        Generate monitoring record from point

        Args:
            point: point

        Returns:
            monitoring record

        """
        return {
            "time": str(datetime.fromtimestamp(point.eventTime)),
            "data": {**point.tags, **point.fields},
        }

    async def stopMonitoring(self) -> None:
        """
        Stop monitoring (cancel all request and stop getting new).
        """
        await super().stopMonitoring()
        await self.session.close()
