"""
Module contains python gc tuning functions

Idea and defaults  - https://habr.com/ru/companies/okko/articles/853406/
"""

import gc
import os

from ..utils.log import logger


def getPositiveIntVar(name: str, default: int) -> int:
    """
    Get environment variable and cast to positive integer number.

    Arguments:
        name {str} -- name of the variable.
        default {int} -- the default value of the variable.

    Returns:
        default {int} -- variable value or the default value of the variable if variable does not set or incorrect.
    """
    try:
        value = int(os.getenv(name, default=str(default)))
    except ValueError:
        logger.error(f"environment variable `{name}` must be an integer, used default {default}")
        return default
    if value < 1:
        logger.error(f"environment variable `{name}` must be > 0, used default {default}")
    return value


def gcTuning():
    """
    Tuning gc threshold for services speed up

    ENVIRONMENT VARIABLES:
        VL_EXPERIMENTS_GC: enable or not tuning
        VL_EXPERIMENTS_GC_THRESHOLD_0: python gc threshold0
        VL_EXPERIMENTS_GC_THRESHOLD_1: python gc threshold1
        VL_EXPERIMENTS_GC_THRESHOLD_2: python gc threshold2
    """
    try:
        tuningEnable = int(os.getenv("VL_EXPERIMENTS_GC", default="1"))
    except ValueError:
        logger.error("environment variable `VL_EXPERIMENTS_GC` must be an integer 0/1")
        return
    if not tuningEnable:
        return
    th0 = getPositiveIntVar("VL_EXPERIMENTS_GC_THRESHOLD_0", 10000)
    th1 = getPositiveIntVar("VL_EXPERIMENTS_GC_THRESHOLD_1", 20)
    th2 = getPositiveIntVar("VL_EXPERIMENTS_GC_THRESHOLD_2", 10)
    gc.set_threshold(th0, th1, th2)
    logger.info(f"gc was tuning, thresholds ({th0}, {th1}, {th2})")
